Unit Kernel;

Interface

Uses
  Windows;

Const
  // Major functions
  IRP_MJ_CREATE                   = $00;
  IRP_MJ_CREATE_NAMED_PIPE        = $01;
  IRP_MJ_CLOSE                    = $02;
  IRP_MJ_READ                     = $03;
  IRP_MJ_WRITE                    = $04;
  IRP_MJ_QUERY_INFORMATION        = $05;
  IRP_MJ_SET_INFORMATION          = $06;
  IRP_MJ_QUERY_EA                 = $07;
  IRP_MJ_SET_EA                   = $08;
  IRP_MJ_FLUSH_BUFFERS            = $09;
  IRP_MJ_QUERY_VOLUME_INFORMATION = $0a;
  IRP_MJ_SET_VOLUME_INFORMATION   = $0b;
  IRP_MJ_DIRECTORY_CONTROL        = $0c;
  IRP_MJ_FILE_SYSTEM_CONTROL      = $0d;
  IRP_MJ_DEVICE_CONTROL           = $0e;
  IRP_MJ_INTERNAL_DEVICE_CONTROL  = $0f;
  IRP_MJ_SHUTDOWN                 = $10;
  IRP_MJ_LOCK_CONTROL             = $11;
  IRP_MJ_CLEANUP                  = $12;
  IRP_MJ_CREATE_MAILSLOT          = $13;
  IRP_MJ_QUERY_SECURITY           = $14;
  IRP_MJ_SET_SECURITY             = $15;
  IRP_MJ_POWER                    = $16;
  IRP_MJ_SYSTEM_CONTROL           = $17;
  IRP_MJ_DEVICE_CHANGE            = $18;
  IRP_MJ_QUERY_QUOTA              = $19;
  IRP_MJ_SET_QUOTA                = $1a;
  IRP_MJ_PNP                      = $1b;
  IRP_MJ_PNP_POWER                = IRP_MJ_PNP;      // Obsolete....
  IRP_MJ_MAXIMUM_FUNCTION         = $1b;

  // Device flags
  DO_VERIFY_VOLUME                   = $00000002;
  DO_BUFFERED_IO                     = $00000004;
  DO_EXCLUSIVE                       = $00000008;
  DO_DIRECT_IO                       = $00000010;
  DO_MAP_IO_BUFFER                   = $00000020;
  DO_DEVICE_HAS_NAME                 = $00000040;
  DO_DEVICE_INITIALIZING             = $00000080;
  DO_SYSTEM_BOOT_PARTITION           = $00000100;
  DO_LONG_TERM_REQUESTS              = $00000200;
  DO_NEVER_LAST_DEVICE               = $00000400;
  DO_SHUTDOWN_REGISTERED             = $00000800;
  DO_BUS_ENUMERATED_DEVICE           = $00001000;
  DO_POWER_PAGABLE                   = $00002000;
  DO_POWER_INRUSH                    = $00004000;
  DO_LOW_PRIORITY_FILESYSTEM         = $00010000;
  DO_SUPPORTS_TRANSACTIONS           = $00040000;
  DO_FORCE_NEITHER_IO                = $00080000;
  DO_VOLUME_DEVICE_OBJECT            = $00100000;
  DO_SYSTEM_SYSTEM_PARTITION         = $00200000;
  DO_SYSTEM_CRITICAL_PARTITION       = $00400000;
  DO_DISALLOW_EXECUTE                = $00800000;

  // Device characteristics
  FILE_REMOVABLE_MEDIA                   = $00000001;
  FILE_READ_ONLY_DEVICE                  = $00000002;
  FILE_FLOPPY_DISKETTE                   = $00000004;
  FILE_WRITE_ONCE_MEDIA                  = $00000008;
  FILE_REMOTE_DEVICE                     = $00000010;
  FILE_DEVICE_IS_MOUNTED                 = $00000020;
  FILE_VIRTUAL_VOLUME                    = $00000040;
  FILE_AUTOGENERATED_DEVICE_NAME         = $00000080;
  FILE_DEVICE_SECURE_OPEN                = $00000100;
  FILE_CHARACTERISTIC_PNP_DEVICE         = $00000800;
  FILE_CHARACTERISTIC_TS_DEVICE          = $00001000;
  FILE_CHARACTERISTIC_WEBDAV_DEVICE      = $00002000;

  // Driver flags
  DRVO_UNLOAD_INVOKED            = $00000001;
  DRVO_LEGACY_DRIVER             = $00000002;
  DRVO_BUILTIN_DRIVER            = $00000004;    // Driver objects for Hal, PnP Mgr
  DRVO_REINIT_REGISTERED         = $00000008;
  DRVO_INITIALIZED               = $00000010;
  DRVO_BOOTREINIT_REGISTERED     = $00000020;
  DRVO_LEGACY_RESOURCES          = $00000040;
  DRVO_BASE_FILESYSTEM_DRIVER    = $00000080;

Function IrpMajorToStr(AMajor:ULONG):WideString;
Function DriverFlagsToStr(AFlags:Cardinal):WideString;
Function DriverFlagToStr(AFlag:Cardinal):WideString;
Function DeviceFlagsToStr(AFlags:Cardinal):WideString;
Function DeviceFlagToStr(AFlag:Cardinal):WideString;
Function DeviceCharacteristicToStr(AChar:Cardinal):WideString;
Function DeviceCharacteristicsToStr(AChars:Cardinal):WideString;

Implementation

Uses
  SysUtils;

Type
  TFlagToStrFunction = Function (AFlag:Cardinal):WideString;

Function _FlagsToStr(AFlagArray: Array Of Cardinal; AFlags:Cardinal; AFunction:TFlagToStrFunction):WideString;
Var
  I : Integer;
  strFlag : WideString;
begin
For I := Low(AFlagArray) To High(AFlagArray) Do
  begin
  strFlag := AFunction(AFlags And AFlagArray[I]);
  If strFlag <> '' Then
    begin
    If Result <> '' Then
      Result := Result + ' ';

    Result := Result + strFlag;
    end;
  end;
end;

Function DeviceCharacteristicsToStr(AChars:Cardinal):WideString;
Var
  CharArray : Array [0..11] OF Cardinal;
begin
CharArray[0] := FILE_REMOVABLE_MEDIA;
CharArray[1] := FILE_READ_ONLY_DEVICE;
CharArray[2] := FILE_FLOPPY_DISKETTE;
CharArray[3] := FILE_WRITE_ONCE_MEDIA;
CharArray[4] := FILE_REMOTE_DEVICE;
CharArray[5] := FILE_DEVICE_IS_MOUNTED;
CharArray[6] := FILE_VIRTUAL_VOLUME;
CharArray[7] := FILE_AUTOGENERATED_DEVICE_NAME;
CharArray[8] := FILE_DEVICE_SECURE_OPEN;
CharArray[9] := FILE_CHARACTERISTIC_PNP_DEVICE;
CharArray[10] := FILE_CHARACTERISTIC_TS_DEVICE;
CharArray[11] := FILE_CHARACTERISTIC_WEBDAV_DEVICE;
Result := _FlagsToStr(CharArray, AChars, DeviceCharacteristicToStr);
end;

Function DeviceFlagsToStr(AFlags:Cardinal):WideString;
Var
  FlagArray : Array [0..20] Of Cardinal;
begin
FlagArray[0] := DO_VERIFY_VOLUME;
FlagArray[1] := DO_BUFFERED_IO;
FlagArray[2] := DO_EXCLUSIVE;
FlagArray[3] := DO_DIRECT_IO;
FlagArray[4] := DO_MAP_IO_BUFFER;
FlagArray[5] := DO_DEVICE_HAS_NAME;
FlagArray[6] := DO_DEVICE_INITIALIZING;
FlagArray[7] := DO_SYSTEM_BOOT_PARTITION;
FlagArray[8] := DO_LONG_TERM_REQUESTS;
FlagArray[9] := DO_NEVER_LAST_DEVICE;
FlagArray[10] := DO_SHUTDOWN_REGISTERED;
FlagArray[11] := DO_BUS_ENUMERATED_DEVICE;
FlagArray[12] := DO_POWER_PAGABLE;
FlagArray[13] := DO_POWER_INRUSH;
FlagArray[14] := DO_LOW_PRIORITY_FILESYSTEM;
FlagArray[15] := DO_SUPPORTS_TRANSACTIONS;
FlagArray[16] := DO_FORCE_NEITHER_IO;
FlagArray[17] := DO_VOLUME_DEVICE_OBJECT;
FlagArray[18] := DO_SYSTEM_SYSTEM_PARTITION;
FlagArray[19] := DO_SYSTEM_CRITICAL_PARTITION;
FlagArray[20] := DO_DISALLOW_EXECUTE;
Result := _FlagsToStr(FlagArray, AFlags, DeviceFlagToStr);
end;

Function DriverFlagsToStr(AFlags:Cardinal):WideString;
Var
  FlagArray : Array [0..7] Of Cardinal;
begin
Result := '';
FlagArray[0] := DRVO_UNLOAD_INVOKED;
FlagArray[1] := DRVO_LEGACY_DRIVER;
FlagArray[2] := DRVO_BUILTIN_DRIVER;
FlagArray[3] := DRVO_REINIT_REGISTERED;
FlagArray[4] := DRVO_INITIALIZED;
FlagArray[5] := DRVO_BOOTREINIT_REGISTERED;
FlagArray[6] := DRVO_LEGACY_RESOURCES;
FlagArray[7] := DRVO_BASE_FILESYSTEM_DRIVER;
Result := _FlagsToStr(FlagArray, AFlags, DriverFlagToStr);
end;

Function DeviceCharacteristicToStr(AChar:Cardinal):WideString;
begin
Case AChar Of
  FILE_REMOVABLE_MEDIA : Result := 'RemovableMedia';
  FILE_READ_ONLY_DEVICE : Result := 'ReadOnly';
  FILE_FLOPPY_DISKETTE : Result := 'Floppy';
  FILE_WRITE_ONCE_MEDIA : Result := 'WriteOnce';
  FILE_REMOTE_DEVICE : Result := 'Remote';
  FILE_DEVICE_IS_MOUNTED : Result := 'Mounted';
  FILE_VIRTUAL_VOLUME : Result := 'VirtualVolume';
  FILE_AUTOGENERATED_DEVICE_NAME : Result := 'AutogeneratedName';
  FILE_DEVICE_SECURE_OPEN : Result := 'SecureOpen';
  FILE_CHARACTERISTIC_PNP_DEVICE : Result := 'PnPDevice';
  FILE_CHARACTERISTIC_TS_DEVICE : Result := 'TsDevice';
  FILE_CHARACTERISTIC_WEBDAV_DEVICE : Result := 'WebDAVDevice';
  Else Result := '';
  end;
end;

Function DeviceFlagToStr(AFlag:Cardinal):WideString;
begin
Case AFlag Of
  DO_VERIFY_VOLUME : Result := 'VerifyVolume';
  DO_BUFFERED_IO : Result := 'BufferedIo';
  DO_EXCLUSIVE : Result := 'Exclusive';
  DO_DIRECT_IO : Result := 'DirectIo';
  DO_MAP_IO_BUFFER : Result := 'MapIoBuffer';
  DO_DEVICE_HAS_NAME : Result := 'HasName';
  DO_DEVICE_INITIALIZING : Result := 'Initializing';
  DO_SYSTEM_BOOT_PARTITION : Result := 'SystemBootPartition';
  DO_LONG_TERM_REQUESTS : Result := 'LongTermRequests';
  DO_NEVER_LAST_DEVICE : Result := 'NeverLastDevice';
  DO_SHUTDOWN_REGISTERED : Result := 'ShutdownRegistered';
  DO_BUS_ENUMERATED_DEVICE : Result := 'BusEnumerated';
  DO_POWER_PAGABLE : Result := 'PowerPagable';
  DO_POWER_INRUSH : Result := 'PowerInrush';
  DO_LOW_PRIORITY_FILESYSTEM : Result := 'LowPriorityFilesystem';
  DO_SUPPORTS_TRANSACTIONS : Result := 'SupportsTransactions';
  DO_FORCE_NEITHER_IO : Result := 'ForceNeitherIo';
  DO_VOLUME_DEVICE_OBJECT : Result := 'VolumeDeviceObject';
  DO_SYSTEM_SYSTEM_PARTITION : Result := 'SystemPartition';
  DO_SYSTEM_CRITICAL_PARTITION : Result := 'CriticalPartition';
  DO_DISALLOW_EXECUTE : Result := 'DisallowExecute';
  Else Result := '';
  end;
end;

Function DriverFlagToStr(AFlag:Cardinal):WideString;
begin
Case AFlag Of
  DRVO_UNLOAD_INVOKED : Result := 'UnloadInvoked';
  DRVO_LEGACY_DRIVER : Result := 'LegacyDriver';
  DRVO_BUILTIN_DRIVER : Result := 'BuiltInDriver';
  DRVO_REINIT_REGISTERED : Result := 'InitRegistered';
  DRVO_INITIALIZED : Result := 'Initialized';
  DRVO_BOOTREINIT_REGISTERED : Result := 'BootReinitRegistered';
  DRVO_LEGACY_RESOURCES : Result := 'LegacyResources';
  DRVO_BASE_FILESYSTEM_DRIVER : Result := 'BaseFileSystemDriver';
  Else Result := '';
  end;
end;

Function IrpMajorToStr(AMajor:ULONG):WideString;
begin
Case AMajor Of
  IRP_MJ_CREATE : Result := 'Create';
  IRP_MJ_CREATE_NAMED_PIPE : Result := 'CreateNamedPipe';
  IRP_MJ_CLOSE : Result := 'Close';
  IRP_MJ_READ : Result := 'Read';
  IRP_MJ_WRITE : Result := 'Write';
  IRP_MJ_QUERY_INFORMATION : Result := 'QueryInformation';
  IRP_MJ_SET_INFORMATION : Result := 'SetInformation';
  IRP_MJ_QUERY_EA : Result := 'QueryEA';
  IRP_MJ_SET_EA : Result := 'SetEA';
  IRP_MJ_FLUSH_BUFFERS : Result := 'FlushBuffers';
  IRP_MJ_QUERY_VOLUME_INFORMATION : Result := 'QueryVolumeInformation';
  IRP_MJ_SET_VOLUME_INFORMATION : Result := 'SetVolumeInformation';
  IRP_MJ_DIRECTORY_CONTROL :  Result := 'DirectoryControl';
  IRP_MJ_FILE_SYSTEM_CONTROL  : Result := 'FileSystemControl';
  IRP_MJ_DEVICE_CONTROL  : Result := 'DeviceControl';
  IRP_MJ_INTERNAL_DEVICE_CONTROL  : Result := 'InternalDeviceControl';
  IRP_MJ_SHUTDOWN : Result := 'Shutdown';
  IRP_MJ_LOCK_CONTROL : Result := 'LockControl';
  IRP_MJ_CLEANUP : Result := 'Cleanup';
  IRP_MJ_CREATE_MAILSLOT : Result := 'CreateMailslot';
  IRP_MJ_QUERY_SECURITY : Result := 'QuerySecurity';
  IRP_MJ_SET_SECURITY : Result := 'SetSecurity';
  IRP_MJ_POWER : Result := 'Power';
  IRP_MJ_SYSTEM_CONTROL : Result := 'SystemControl';
  IRP_MJ_DEVICE_CHANGE : Result := 'DeviceChange';
  IRP_MJ_QUERY_QUOTA : Result := 'QueryQuota';
  IRP_MJ_SET_QUOTA : Result := 'setQuota';
  IRP_MJ_PNP : Result := 'Pnp';
  end;
end;


End.
