Unit LogSettings;

Interface

Uses
  Classes, HashTable;

Type
  TDeviceLogSettings = Record
    IncludeType : Boolean;
    IncludeDiskDevice : Boolean;

    IncludeUpperDevicesCount : Boolean;
    IncludeUpperDevices : Boolean;
    IncludeLowerDevicesCount : Boolean;
    IncludeLowerDevices : Boolean;

    IncludeFlags : Boolean;
    IncludeFlagsStr : Boolean;
    IncludeCharacteristics : Boolean;
    IncludeCharacteristicsStr : Boolean;

    IncludePnPInformation : Boolean;
    IncludeFriendlyName : Boolean;
    IncludeDescription : Boolean;
    IncludeVendor : Boolean;
    IncludeEnumerator : Boolean;
    IncludeLocation : Boolean;
    IncludeClass : Boolean;
    IncludeClassGuid : Boolean;
    end;
  PDeviceLogSettings = ^TDeviceLogSettings;

  TDriverLogSettings = Record
    IncludeFileName : Boolean;
    IncludeImageBase : Boolean;
    IncludeImageSize : Boolean;
    IncludeDriverEntry : Boolean;
    IncludeDriverUnload : Boolean;
    IncludeStartIo : Boolean;
    IncludeFlags : Boolean;
    IncludeFlagsStr : Boolean;
    IncludeMajorFunctions : Boolean;
    IncludeNumberOfDevices : Boolean;
    IncludeDevices : Boolean;
    end;
  PDriverLogSettings = ^TDriverLogSettings;

  TGeneralLogSettings = Record
    IncludeVTHeader : Boolean;
    IncludeOSVersion : Boolean;
    end;
  PGeneralLogSettings = ^TGeneralLogSettings;

  TLogSettings = Class
    Private
      FDriverTable : TAddressHashTable;
      FDeviceTable : TAddressHashTable;
      FIncludedDrivers : TList;
    Public
      General : TGeneralLogSettings;
      DriverSettings : TDriverLogSettings;
      DeviceSettings : TDeviceLogSettings;
      Procedure IncludeDriver(AAddress:Pointer);
      Function DriverIncluded(AAddress:Pointer):Boolean;
      Procedure Clear;
      Constructor Create(ADriverTable:TAddressHashTable; ADeviceTable:TAddressHashTable); Reintroduce;
      Destructor Destroy; Override;
    end;

Implementation

Procedure TLogSettings.IncludeDriver(AAddress:Pointer);
begin
FIncludedDrivers.Add(AAddress);
end;

Procedure TLogSettings.Clear;
begin
FIncludedDrivers.Clear;
end;

Constructor TLogSettings.Create(ADriverTable:TAddressHashTable; ADeviceTable:TAddressHashTable);
begin
Inherited Create;
FDriverTable := ADriverTable;
FDeviceTable := ADeviceTable;
FIncludedDrivers := TList.Create;
end;

Destructor TLogSettings.Destroy;
begin
FIncludedDrivers.Free;
Inherited Destroy;
end;

Function TLogSettings.DriverIncluded(AAddress:Pointer):Boolean;
begin
Result := FIncludedDrivers.IndexOf(AAddress) > -1;
end;

End.
