
#include <ntifs.h>
#include "preprocessor.h"
#include "utils-mem.h"
#include "utils-stack.h"




NTSTATUS StackCreate(POOL_TYPE PoolType, PUTILS_STACK *Stack)
{
   PUTILS_STACK tmpStack;
   NTSTATUS status = STATUS_UNSUCCESSFUL;
   DEBUG_ENTER_FUNCTION("PoolType=%u; Stack=0x%p", PoolType, Stack);

   tmpStack = (PUTILS_STACK)HeapMemoryAlloc(PoolType, sizeof(UTILS_STACK));
   if (tmpStack != NULL) {
      tmpStack->PoolType = PoolType;
      tmpStack->Top = (PUTILS_STACK_ITEM)&tmpStack->Top;
      *Stack = tmpStack;
      status = STATUS_SUCCESS;
   } else status = STATUS_INSUFFICIENT_RESOURCES;

   DEBUG_EXIT_FUNCTION("0x%x, *Stack=0x%p", status, *Stack);
   return status;
}


VOID StackPushNoAlloc(PUTILS_STACK Stack, PUTILS_STACK_ITEM Item)
{
   DEBUG_ENTER_FUNCTION("Stack=0x%p; Item=0x%p", Stack, Item);

   Item->Next = InterlockedExchangePointer(&Stack->Top, Item);

   DEBUG_EXIT_FUNCTION_VOID();
   return;
}

NTSTATUS StackPush(PUTILS_STACK Stack, PVOID Value)
{
   PUTILS_STACK_ITEM item = NULL;
   NTSTATUS status = STATUS_UNSUCCESSFUL;
   DEBUG_ENTER_FUNCTION("Stack=0x%p; Value=0x%p", Stack, Value);

   item = (PUTILS_STACK_ITEM)HeapMemoryAlloc(Stack->PoolType, sizeof(UTILS_STACK_ITEM));
   if (item != NULL) {
      item->Value = Value;
      StackPushNoAlloc(Stack, item);
      status = STATUS_SUCCESS;
   } else status = STATUS_INSUFFICIENT_RESOURCES;

   DEBUG_EXIT_FUNCTION("0x%x", status);
   return status;
}

PUTILS_STACK_ITEM StackPopNoFree(PUTILS_STACK Stack)
{
   PUTILS_STACK_ITEM ret = NULL;
   DEBUG_ENTER_FUNCTION("Stack=0x%p", Stack);

   ret = InterlockedExchangePointer(&Stack->Top, Stack->Top->Next);
   if (ret == (PUTILS_STACK_ITEM)&Stack->Top)
      ret = NULL;

   DEBUG_EXIT_FUNCTION("0x%p", ret);
   return ret;
}



PVOID StackPop(PUTILS_STACK Stack)
{
   PVOID ret = NULL;
   PUTILS_STACK_ITEM item = NULL;
   DEBUG_ENTER_FUNCTION("Stack=0x%p", Stack);

   item = StackPopNoFree(Stack);
   if (item != NULL) {
      ret = item->Value;
      HeapMemoryFree(item);
   }

   DEBUG_EXIT_FUNCTION("0x%p", ret);
   return ret;
}


BOOLEAN StackEmpty(PUTILS_STACK Stack)
{
   BOOLEAN ret = FALSE;
   DEBUG_ENTER_FUNCTION("Stack=0x%p", Stack);

   ret = (Stack->Top == (PUTILS_STACK_ITEM)&Stack->Top);

   DEBUG_EXIT_FUNCTION("%u", ret);
   return ret;
}


VOID StackDestroy(PUTILS_STACK Stack)
{
   DEBUG_ENTER_FUNCTION("Stack=0x%p", Stack);

   HeapMemoryFree(Stack);

   DEBUG_EXIT_FUNCTION_VOID();
   return;
}
